@@ -1,6 +1,6 @@
 /* runner.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2012 Ralf Hoffmann.
+ * Copyright (C) 2012-2020 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -136,7 +136,8 @@ int KnownPrograms::read( const std::string &config_file )
 
 RunnerUI::RunnerUI( int argc, char **argv ) : m_argc( argc ),
                                               m_argv( argv ),
-                                              m_completion_mode( HISTORY_MODE )
+                                              m_completion_mode( HISTORY_MODE ),
+                                              m_show_run_dialog( false )
 {
     m_aguix = new AGUIX( argc, argv, "Runner" );
     m_aguix->initX();
@@ -244,7 +245,8 @@ void RunnerUI::runProgramDialog()
     m_win = new AWindow( m_aguix,
                          0, 0,
                          400, 400,
-                         0, "Run program" );
+                         "Run program",
+                         AWindow::AWINDOW_NORMAL );
     m_win->create();
 
     m_co1 = m_win->setContainer( new AContainer( m_win, 1, 4 ), true );
@@ -253,12 +255,12 @@ void RunnerUI::runProgramDialog()
     AContainer *co1_2 = m_co1->add( new AContainer( m_win, 2, 1 ), 0, 0 );
     co1_2->setMaxSpace( 5 );
     co1_2->setBorderWidth( 0 );
-    co1_2->add( new Text( m_aguix, 0, 0, "Enter program:", 1 ),
+    co1_2->add( new Text( m_aguix, 0, 0, "Enter program:" ),
                 0, 0, AContainer::CO_FIX );
     m_sg = static_cast<StringGadget*>( co1_2->add( new StringGadget( m_aguix, 0, 0, 100, "", 1 ),
                                                    1, 0, AContainer::CO_INCW ) );
     
-    m_mode_cyb = static_cast< CycleButton * >( m_co1->add( new CycleButton( m_aguix, 0, 0, 100, 1, 0, 0 ),
+    m_mode_cyb = static_cast< CycleButton * >( m_co1->add( new CycleButton( m_aguix, 0, 0, 100, 0 ),
                                                            0, 1, AContainer::CO_INCW ) );
 
     m_mode_cyb->addOption( "History (TAB: switch to all programs)" );
@@ -419,20 +421,32 @@ void RunnerUI::runProgramDialog()
             }
             m_aguix->ReplyMessage( msg );
         }
-    }
 
-    if ( endmode == 1 ) {
-        std::string cmd = m_sg->getText();
+        if ( endmode == 1 ) {
+            std::string cmd = m_sg->getText();
+            bool store = true;
 
-        if ( ! cmd.empty() ) {
-            if ( cmd[0] == ' ' ) {
-                cmd.erase( 0, 1 );
-            } else {
-                m_history->removeEntry( cmd );
-                m_history->pushFrontEntry( cmd );
+            if ( ! cmd.empty() ) {
+                if ( cmd[0] == ' ' ) {
+                    cmd.erase( 0, 1 );
+                    store = false;
+                }
+
+                runCommand( cmd );
+
+                if ( store ) {
+                    if ( commandExists( cmd ) ) {
+                        m_history->removeEntry( cmd );
+                        m_history->pushFrontEntry( cmd );
+                    } else {
+                        Requester req( m_aguix );
+                        req.request( "Runner error",
+                                     "Command not found",
+                                     "Close" );
+                        endmode = 0;
+                    }
+                }
             }
-
-            runCommand( cmd );
         }
     }
 
@@ -451,83 +465,87 @@ int RunnerUI::mainLoop()
     cfg += "/program_list";
     m_programs.read( cfg );
 
-    std::list<PopUpMenu::PopUpEntry> m1;
-    PopUpMenu::PopUpEntry e1;
-    e1.name = "Start command:";
-    e1.type = PopUpMenu::HEADER;
-    m1.push_back( e1 );
-    e1.type = PopUpMenu::HLINE;
-    m1.push_back( e1 );
+    if ( m_show_run_dialog == false ) {
+        std::list<PopUpMenu::PopUpEntry> m1;
+        PopUpMenu::PopUpEntry e1;
+        e1.name = "Start command:";
+        e1.type = PopUpMenu::HEADER;
+        m1.push_back( e1 );
+        e1.type = PopUpMenu::HLINE;
+        m1.push_back( e1 );
 
-    int pos = 0;
-    for ( auto it1 = m_programs.getList().begin();
-          it1 != m_programs.getList().end();
-          it1++ ) {
-	e1.type = PopUpMenu::NORMAL;
+        int pos = 0;
+        for ( auto it1 = m_programs.getList().begin();
+              it1 != m_programs.getList().end();
+              it1++ ) {
+            e1.type = PopUpMenu::NORMAL;
 
-        e1.name = it1->first;
-	e1.id = pos++;
-	m1.push_back( e1 );
-    }
-
-    e1.type = PopUpMenu::HLINE;
-    m1.push_back( e1 );
-
-    e1.type = PopUpMenu::NORMAL;
-
-    e1.name = "Run...";
-    e1.id = -1;
-    m1.push_back( e1 );
-
-    PopUpMenu *command_menu = new PopUpMenu( m_aguix, m1 );
-    command_menu->create();
-
-    int mouse_x, mouse_y;
-
-    m_aguix->queryRootPointer( &mouse_x, &mouse_y );
-
-    int root_x, root_y,
-        root_width,
-        root_height;
-
-    m_aguix->getLargestDimensionOfCurrentScreen( &root_x, &root_y,
-                                                 &root_width, &root_height );
-
-    if ( mouse_x + command_menu->getWidth() >= root_x + root_width ||
-         mouse_y + command_menu->getHeight() >= root_y + root_height ) {
-        command_menu->show( a_max( 0, mouse_x - command_menu->getWidth() ),
-                            a_max( 0, mouse_y - command_menu->getHeight() ),
-                            PopUpMenu::POPUP_IN_POSITION );
-    } else {
-        command_menu->show();
-    }
-
-    AGMessage *msg;
-    int endmode = 0;
-
-    command_menu->setXFocus();
-
-    for ( ; endmode == 0; ) {
-        msg = m_aguix->WaitMessage( NULL );
-        if ( msg != NULL ) {
-            switch ( msg->type ) {
-		case AG_POPUPMENU_CLICKED:
-		    if ( msg->popupmenu.menu == command_menu ) {
-			handlePopupMsg( msg );
-                        endmode = -1;
-		    }
-                    break;
-		case AG_POPUPMENU_CLOSED:
-		    if ( msg->popupmenu.menu == command_menu ) {
-                        endmode = -1;
-                    }
-                    break;
-            }
-            m_aguix->ReplyMessage( msg );
+            e1.name = it1->first;
+            e1.id = pos++;
+            m1.push_back( e1 );
         }
-    }
 
-    delete command_menu;
+        e1.type = PopUpMenu::HLINE;
+        m1.push_back( e1 );
+
+        e1.type = PopUpMenu::NORMAL;
+
+        e1.name = "Run...";
+        e1.id = -1;
+        m1.push_back( e1 );
+
+        PopUpMenu *command_menu = new PopUpMenu( m_aguix, m1 );
+        command_menu->create();
+
+        int mouse_x, mouse_y;
+
+        m_aguix->queryRootPointer( &mouse_x, &mouse_y );
+
+        int root_x, root_y,
+            root_width,
+            root_height;
+
+        m_aguix->getLargestDimensionOfCurrentScreen( &root_x, &root_y,
+                                                     &root_width, &root_height );
+
+        if ( mouse_x + command_menu->getWidth() >= root_x + root_width ||
+             mouse_y + command_menu->getHeight() >= root_y + root_height ) {
+            command_menu->show( a_max( 0, mouse_x - command_menu->getWidth() ),
+                                a_max( 0, mouse_y - command_menu->getHeight() ),
+                                PopUpMenu::POPUP_IN_POSITION );
+        } else {
+            command_menu->show();
+        }
+
+        AGMessage *msg;
+        int endmode = 0;
+
+        command_menu->setXFocus();
+
+        for ( ; endmode == 0; ) {
+            msg = m_aguix->WaitMessage( NULL );
+            if ( msg != NULL ) {
+                switch ( msg->type ) {
+                    case AG_POPUPMENU_CLICKED:
+                        if ( msg->popupmenu.menu == command_menu ) {
+                            handlePopupMsg( msg );
+                            endmode = -1;
+                        }
+                        break;
+                    case AG_POPUPMENU_CLOSED:
+                        if ( msg->popupmenu.menu == command_menu ) {
+                            endmode = -1;
+                        }
+                        break;
+                }
+                m_aguix->ReplyMessage( msg );
+            }
+        }
+
+        delete command_menu;
+    } else {
+        runProgramDialog();
+    }
 
     return 0;
 }
@@ -580,6 +598,47 @@ private:
     std::list< std::string > &m_l;
 };
 
+std::list< std::string > RunnerUI::getCommands() const
+{
+    const char *path = getenv( "PATH" );
+
+    if ( ! path ) {
+        return {};
+    }
+
+    std::vector< std::string > l;
+
+    AGUIXUtils::split_string( l, path, ':' );
+
+    std::list< std::string > all_commands;
+    std::list< std::string > unique_commands;
+
+    MyDirWalk dircb( all_commands );
+
+    for ( auto it1 = l.begin();
+          it1 != l.end();
+          it1++ ) {
+
+        NWC::FSEntry p( *it1 );
+
+        if ( p.isDir( true ) ) {
+            NWC::Dir d( p );
+
+            d.readDir();
+
+            d.walk( dircb );
+        }
+    }
+
+    all_commands.sort();
+
+    std::unique_copy( all_commands.begin(),
+                      all_commands.end(),
+                      std::back_inserter( unique_commands ) );
+
+    return unique_commands;
+}
+
 void RunnerUI::findCommands()
 {
     if ( ! m_commands_in_path.empty() ) return;
@@ -589,38 +648,7 @@ void RunnerUI::findCommands()
         m_aguix->Flush();
     }
 
-    const char *path = getenv( "PATH" );
-
-    if ( path ) {
-        std::vector< std::string > l;
-
-        AGUIXUtils::split_string( l, path, ':' );
-
-        std::list< std::string > all_commands;
-
-        MyDirWalk dircb( all_commands );
-
-        for ( auto it1 = l.begin();
-              it1 != l.end();
-              it1++ ) {
-
-            NWC::FSEntry p( *it1 );
-
-            if ( p.isDir( true ) ) {
-                NWC::Dir d( p );
-
-                d.readDir();
-
-                d.walk( dircb );
-            }
-        }
-
-        all_commands.sort();
-
-        std::unique_copy( all_commands.begin(),
-                          all_commands.end(),
-                          std::back_inserter( m_commands_in_path ) );
-    }
+    m_commands_in_path = getCommands();
 
     if ( m_win != NULL ) {
         m_win->unsetCursor();
@@ -680,3 +708,31 @@ void RunnerUI::switchCompletionMode()
                             s.length() );
     }
 }
+
+void RunnerUI::setShowRunDialog( bool nv )
+{
+    m_show_run_dialog = nv;
+}
+
+bool RunnerUI::commandExists( const std::string &cmd )
+{
+    if ( m_commands_in_path.empty() ) {
+        m_commands_in_path = getCommands();
+    }
+
+    std::vector< std::string > l;
+
+    AGUIXUtils::split_string( l, cmd, ' ' );
+
+    if ( l.empty() ) return false;
+
+    if ( l[0].empty() ) return false;
+
+    for ( auto &path_cmd : m_commands_in_path ) {
+        if ( path_cmd == l[0] ) {
+            return true;
+        }
+    }
+
+    return false;
+}
